#include <time.h>
#include <cam_common.h>

#define ADD_E2STR_ENTRY(x) {x, #x}

const char *state2str(camd_state_t state)
{
	struct {
		camd_state_t state;
		const char *str;
	} state2str_lookup[] = {
		ADD_E2STR_ENTRY(STATE_OFF),
		ADD_E2STR_ENTRY(STATE_ON),
		ADD_E2STR_ENTRY(STATE_AUTO),
		ADD_E2STR_ENTRY(STATE_NONE),
	};
	DEBUG_ASSERT(state >= STATE_OFF || state < STATE_LAST); 

	int i;
	for (i = 0; i < STATE_LAST; i++) {
		if (state2str_lookup[i].state == state)
			return state2str_lookup[i].str;
	}
	return NULL;
}

#ifdef DEBUG
#define MAX_MODULES 32
#define MODULE_MAGIC_START 0xDEADBEEF

typedef struct {
	int magic;
	const char *name;
} module_desc_t;

typedef struct {
	module_desc_t modules_table[MAX_MODULES];
	int magic_template;
	int count;
	int initialized;
} module_data_t;

static module_data_t module_data_singleton;

void module_register(const char *name, int *magic_ref)
{
	module_data_t *d = &module_data_singleton;
	if (!d->initialized) {
		d->magic_template = MODULE_MAGIC_START;
		d->initialized = 1;
	}

	DEBUG_ASSERT(magic_ref);

	/* init only if zero */
	if (*magic_ref)
		return;

	d->modules_table[d->count].name = name;
	d->modules_table[d->count].magic = d->magic_template;
	*magic_ref = d->magic_template;
	d->magic_template++;
	d->count++;
}

static module_desc_t *module_get(module_data_t * d, int magic)
{
	int i;
	for (i = 0; i < d->count; i++) {
		if (d->modules_table[i].magic == magic)
			return &d->modules_table[i];
	}
	return NULL;
}

void module_magic_alert(int expected_magic, int found_magic)
{
	module_data_t *d = &module_data_singleton;
	module_desc_t *e = module_get(d, expected_magic);
	module_desc_t *f = module_get(d, found_magic);
	printf( "fatal: cannot verify module magic number.\n"
			"expected magic is 0x%x for '%s',"
			"found magic is 0x%x describes type '%s'\n",
			expected_magic, e ? e->name : "unknown",
			found_magic, f ? f->name : "unknown");
}
#endif
