/* GStreamer
 *
 * Copyright (C) 2002 Ronald Bultje <rbultje@ronald.bitfreak.net>
 *               2006 Edgard Lima <edgard.lima@indt.org.br>
 *
 * v4l2src.h - system calls
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __V4L2SRC_CALLS_H__
#define __V4L2SRC_CALLS_H__

#include "gstv4l2src.h"
#include "v4l2_calls.h"
#include "PalmOmxjpegEncExif.h"

gboolean gst_v4l2src_get_capture(GstV4l2Src * v4l2src);
gboolean gst_v4l2src_set_capture(GstV4l2Src * v4l2src,
				 guint32 pixelformat,
				 guint32 width, guint32 height,
				 guint32 fps_n, guint32 fps_d);

gboolean still_capture_buff_init(GstV4l2Src * v4l2src, GstCaps *caps, gint fd);
gboolean gst_v4l2src_capture_start(GstV4l2Src * v4l2src);
GstFlowReturn gst_v4l2src_grab_still_frame(GstV4l2Src * v4l2src);
GstFlowReturn gst_v4l2src_grab_video_frame(GstV4l2Src * v4l2src, GstBuffer **buf);
gboolean gst_v4l2src_capture_stop(GstV4l2Src * v4l2src);
gboolean gst_v4l2src_capture_deinit(GstV4l2Src * v4l2src);
gboolean gst_v4l2src_fill_format_list(GstV4l2Src * v4l2src);
gboolean gst_v4l2src_clear_format_list(GstV4l2Src * v4l2src);
gboolean gst_v4l2src_get_still_capture(GstV4l2Src * v4l2src);
gboolean gst_v4l2src_set_still_capture(GstV4l2Src * v4l2src);
gboolean gst_v4l2src_capture_still_done(GstV4l2Src * v4l2src);
gboolean queue_still_buffer(GstV4l2Src *v4l2src);

void still_capture_event_notify(GstV4l2Src * v4l2src);

void gst_v4l2src_send_appmsg(GstV4l2Src * v4l2src, gint errCode);

GstFlowReturn queue_available_preview_buffers(GstV4l2Src *v4l2src);

#define GST_V4L2_SET_ACTIVE(element) (element)->buffer = GINT_TO_POINTER (-1)
#define GST_V4L2_SET_INACTIVE(element) (element)->buffer = NULL

#define DEFAULT_STILL_CAPTURE_WIDTH  2048
#define DEFAULT_STILL_CAPTURE_HEIGHT 1536

#define VGA_PREVIEW
#ifdef VGA_PREVIEW
#define DEFAULT_VIDEO_WIDTH 640
#define DEFAULT_VIDEO_HEIGHT 480
#else
#define DEFAULT_VIDEO_WIDTH 480
#define DEFAULT_VIDEO_HEIGHT 320
#endif

#define PREVIEW_FRAME_SIZE_BYTES (DEFAULT_VIDEO_WIDTH * DEFAULT_VIDEO_HEIGHT * 2)

#ifndef DEBUG
/* compile out LOG and DEBUG logging for performance reasons */
#undef GST_LOG
#undef GST_DEBUG
#define GST_LOG(args...)
#define GST_DEBUG(args...)
#endif /* DEBUG */

#endif /* __V4L2SRC_CALLS_H__ */
